-- ===================================================
-- Constants
-- ===================================================

local ADDON_NAME, Mocksamdi = ...;
local ADDON_VERSION = Mocksamdi.VERSION;
local ADDON_AUTHOR = Mocksamdi.ADDON_AUTHOR;

local FONT = "Fonts\\FRIZQT__.TTF";

-- ===================================================
-- Variables
-- ===================================================

local configFrame = CreateFrame("FRAME"); -- config panel
local checkboxes = {
	MocksamdiPlayAudio = nil;
	MocksamdiShowStartup = nil;
	MocksamdiShowImage = nil;
	MocksamdiDebug = nil;
	-- Sound
	MocksamdiInDungeon = nil;
	MocksamdiInRaid = nil;
	MocksamdiInArena = nil;
	MocksamdiInBattleground = nil;
	-- Model
	MocksamdiShowInDungeon = nil;
	MocksamdiShowInRaid = nil;
	MocksamdiShowInArena = nil;
	MocksamdiShowInBattleground = nil;
};

-- ===================================================
-- Local Functions
-- ===================================================

-- Make a checkbox
local makeCheckbox = function (var, text, placement, indent, useDefault)
	local cb = CreateFrame("CheckButton", "cb_".. var .. "_GlobalName", configFrame, "OptionsCheckButtonTemplate");
	cb:SetPoint("TOPLEFT", indent, placement);
	_G["cb_" .. var .. "_GlobalNameText"]:SetText(text);
	cb:SetChecked(_G[var]);
	if useDefault then
		cb:SetScript("OnClick",
			function ()
				_G[var] = cb:GetChecked();
				Mocksamdi:printDebug("Set " .. var .. ": " .. Mocksamdi:s(_G[var]));
			end
		);
	end
	return cb;
end

-- Set the disabled states of the visual checkboxes
local setDisabled = function ()
	checkboxes.MocksamdiShowInDungeon:SetEnabled(MocksamdiShowImage);
	checkboxes.MocksamdiShowInRaid:SetEnabled(MocksamdiShowImage);
	checkboxes.MocksamdiShowInArena:SetEnabled(MocksamdiShowImage);
	checkboxes.MocksamdiShowInBattleground:SetEnabled(MocksamdiShowImage);
end

-- ===================================================
-- Global Functions
-- ===================================================

-- Init Config
function Mocksamdi:initConfig ()
	-- TODO: This is trash, do better
	local placement = -10;
	configFrame.name = ADDON_NAME;

	-- Title
	configFrame.text = configFrame:CreateFontString(nil, "ARTWORK");
	configFrame.text:SetFont(FONT, 20, "OUTLINE");
	configFrame.text:SetPoint("TOPLEFT", 10, placement);
	configFrame.text:SetText(ADDON_NAME);

	-- Version
	configFrame.textVer = configFrame:CreateFontString(nil, "ARTWORK");
	configFrame.textVer:SetFont(FONT, 11);
	configFrame.textVer:SetPoint("TOPLEFT", 400, placement);
	configFrame.textVer:SetText("Version: " .. ADDON_VERSION);

	-- Author
	configFrame.textAuth = configFrame:CreateFontString(nil, "ARTWORK");
	configFrame.textAuth:SetFont(FONT, 11);
	configFrame.textAuth:SetPoint("TOPLEFT", 400, placement - 14);
	configFrame.textAuth:SetText("Author: " .. ADDON_AUTHOR);

	placement = placement - 25;

	-- MocksamdiPlayAudio (Enable Addon)
	checkboxes.MocksamdiPlayAudio = makeCheckbox("MocksamdiPlayAudio", "Enable " .. ADDON_NAME, placement, 10, true);
	placement = placement - 25;

	-- MocksamdiShowStartup
	checkboxes.MocksamdiShowStartup = makeCheckbox("MocksamdiShowStartup", "Show addon startup text", placement, 10, true);
	placement = placement - 25;

	-- MocksamdiShowImage
	checkboxes.MocksamdiShowImage = makeCheckbox("MocksamdiShowImage", "Show Bwonsamdi on death (global)", placement, 10, false);
	checkboxes.MocksamdiShowImage:SetScript("OnClick",
		function ()
			MocksamdiShowImage = checkboxes.MocksamdiShowImage:GetChecked();
			Mocksamdi:printDebug("Set MocksamdiShowImage: " .. Mocksamdi:s(MocksamdiShowImage));
			setDisabled();
		end
	);
	placement = placement - 25;

	-- start destinations
	configFrame.soundText = configFrame:CreateFontString(nil, "ARTWORK");
	configFrame.soundText:SetFont(FONT, 11);
	configFrame.soundText:SetPoint("TOPLEFT", 13, placement - 8);
	configFrame.soundText:SetText("Control Instances:");
	placement = placement - 25;

	-- MocksamdiInDungeon
	checkboxes.MocksamdiInDungeon = makeCheckbox("MocksamdiInDungeon", "In dungeons", placement, 30, true);
	checkboxes.MocksamdiShowInDungeon = makeCheckbox("MocksamdiShowInDungeon", "Show Bwonsamdi", placement, 150, true);
	placement = placement - 25;

	-- MocksamdiInRaid
	checkboxes.MocksamdiInRaid = makeCheckbox("MocksamdiInRaid", "In raids", placement, 30, true);
	checkboxes.MocksamdiShowInRaid = makeCheckbox("MocksamdiShowInRaid", "Show Bwonsamdi", placement, 150, true);
	placement = placement - 25;

	-- MocksamdiInArena
	checkboxes.MocksamdiInArena = makeCheckbox("MocksamdiInArena", "In arenas", placement, 30, true);
	checkboxes.MocksamdiShowInArena = makeCheckbox("MocksamdiShowInArena", "Show Bwonsamdi", placement, 150, true);
	placement = placement - 25;

	-- MocksamdiInBattleground
	checkboxes.MocksamdiInBattleground = makeCheckbox("MocksamdiInBattleground", "In battlegrounds", placement, 30, true);
	checkboxes.MocksamdiShowInBattleground = makeCheckbox("MocksamdiShowInBattleground", "Show Bwonsamdi", placement, 150, true);
	placement = placement - 25;

	-- MocksamdiDebug
	local debugCB = CreateFrame("CheckButton", "debugCB_GlobalName", configFrame, "OptionsCheckButtonTemplate");
	debugCB:SetPoint("BOTTOMRIGHT", -47, 10);
	debugCB_GlobalNameText:SetText("Debug");
	debugCB:SetChecked(MocksamdiDebug);
	debugCB:SetScript("OnClick",
		function ()
			MocksamdiDebug = debugCB:GetChecked();
			Mocksamdi:printDebug("Set Debug: " .. Mocksamdi:s(MocksamdiDebug));
	 	end
	);
	checkboxes.MocksamdiDebug = debugCB;

	-- Refresh on show so the checkboxes are up to date
	configFrame:SetScript("OnShow",
		function (self, event, ...)
			Mocksamdi:refreshConfig();
		end
	);

	setDisabled();

	-- Append to options panel
	InterfaceOptions_AddCategory(configFrame);
end

-- Refresh the config values
function Mocksamdi:refreshConfig ()
	Mocksamdi:printDebug("Refreshing");
	checkboxes.MocksamdiPlayAudio:SetChecked(MocksamdiPlayAudio);
	checkboxes.MocksamdiShowStartup:SetChecked(MocksamdiShowStartup);
	checkboxes.MocksamdiShowImage:SetChecked(MocksamdiShowImage);
	checkboxes.MocksamdiDebug:SetChecked(MocksamdiDebug);
	-- Sound
	checkboxes.MocksamdiInDungeon:SetChecked(MocksamdiInDungeon);
	checkboxes.MocksamdiInRaid:SetChecked(MocksamdiInRaid);
	checkboxes.MocksamdiInArena:SetChecked(MocksamdiInArena);
	checkboxes.MocksamdiInBattleground:SetChecked(MocksamdiInBattleground);
	-- Model
	checkboxes.MocksamdiShowInDungeon:SetChecked(MocksamdiShowInDungeon);
	checkboxes.MocksamdiShowInRaid:SetChecked(MocksamdiShowInRaid);
	checkboxes.MocksamdiShowInArena:SetChecked(MocksamdiShowInArena);
	checkboxes.MocksamdiShowInBattleground:SetChecked(MocksamdiShowInBattleground);

	setDisabled();
end
