-- ===================================================
-- Constants
-- ===================================================

local ADDON_NAME, Mocksamdi = ...;

local MODEL_ID = 1802805; -- Ghost Bwonsamdi
local MODEL_SCALE = 0.7;
local MODEL_FADE = 0.7;

local ANIMATIONS = {
  -- 60 -- talk
  70, -- laugh
  77, -- face palm
  78, -- chicken
  79, -- knee
  80  -- clap
};

-- ===================================================
-- Variables
-- ===================================================

local caseFrame = CreateFrame("Frame", nil, UIParent)
local modelFrame = CreateFrame("PlayerModel", nil, caseFrame)

-- ===================================================
-- Local Functions
-- ===================================================

-- Check instance type
local checkIfInstanceOk = function ()
  local inInstance, instanceType = IsInInstance();

  -- no instance
  if not inInstance or instanceType == "none" or instanceType == nil then
		return true;
	end

  -- bg
	if instanceType == "pvp" and MocksamdiShowInBattleground then
		return true;
	end

	-- arena
	if instanceType == "arena" and MocksamdiShowInArena then
		return true;
	end

	-- dungeon
	if instanceType == "party" and MocksamdiShowInDungeon then
		return true;
	end

	-- raid
	if instanceType == "raid" and MocksamdiShowInRaid then
		return true;
	end

	return false;
end

-- draw case for model
local drawCase = function ()
  caseFrame:SetPoint("Center", - 300, 100);
  caseFrame:SetWidth(512);
  caseFrame:SetHeight(712);
  caseFrame:SetAlpha(1);
  caseFrame:Hide();
end

-- draw model
local drawModel = function ()
  modelFrame:SetAlpha(MODEL_FADE);
  modelFrame:SetModelScale(MODEL_SCALE);
  modelFrame:SetAllPoints(drawCase);
  modelFrame:SetModel(MODEL_ID);
  modelFrame:SetCamera(1);

  modelFrame:SetAnimation(0);

  modelFrame:Show();
end

-- Fade model out
function fadeModel (fade)
  if fade < .2 then
    caseFrame:Hide();
    modelFrame:SetAnimation(0);
    return;
  end

  modelFrame:SetAlpha(fade);

  -- fade model out over a few seconds
  C_Timer.After(1, function()
    fadeModel(fade - .2);
  end)
end

-- hide model
local hideModel = function ()
  Mocksamdi:printDebug("Hiding model");
  fadeModel(MODEL_FADE - .2);
end

-- ===================================================
-- Global Functions
-- ===================================================

-- init model
function Mocksamdi:initModel ()
  drawCase();
  drawModel();
end

-- show model and play animation
function Mocksamdi:showModel ()
  if not MocksamdiShowImage and not checkIfInstanceOk() then
    return;
  end

  caseFrame:Show();
  drawModel()

  local animationCount = table.getn(ANIMATIONS);
  local animationIndex = math.random(1, animationCount);
  modelFrame:SetAnimation(ANIMATIONS[animationIndex]);
  modelFrame:SetScript("OnAnimFinished",
    function()
      modelFrame:SetAnimation(ANIMATIONS[animationIndex]);
    end
  );

  Mocksamdi:printDebug("Showing model with animation " .. ANIMATIONS[animationIndex]);

  -- timeout then hide
  C_Timer.After(5,
    function()
      modelFrame:SetScript("OnAnimFinished", nil);
      hideModel();
    end
  );
end
